/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.integration.animation_motion;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import org.robokind.api.animation.utils.ChannelNode;
import org.robokind.api.animation.utils.ChannelNode.DefaultChannelDefinition;
import org.robokind.api.animation.utils.ChannelsParameterSource;
import org.robokind.api.animation.utils.DefaultChannelNode;
import org.robokind.api.common.position.NormalizedDouble;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.Robot.JointId;
import org.robokind.api.motion.Robot.RobotPositionMap;
import org.robokind.api.motion.Joint;
import org.robokind.api.motion.jointgroup.JointGroup;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class ChannelRobotParameters implements ChannelsParameterSource{
    private Robot myRobot;
    private ChannelNode myNode;
    private Map<Integer, String> myChannelNames;
    private Map<Integer, Double> myDefaultPositions;
    
    public synchronized void setRobot(Robot robot){
        myRobot = robot;
        myChannelNames = buildChannelNames(myRobot);
        myDefaultPositions = buildDefaultPositions(myRobot);
    }
    
    public synchronized void setJointGroup(JointGroup group){
        myNode = buildChannelTree(group);
    }
    
    @Override
    public Map<Integer, String> getChannelNames() {
        return myChannelNames;
    }

    @Override
    public String getChannelName(int id) {
        Map<Integer,String> map = getChannelNames();
        return map == null ? null : map.get(id);
    }

    @Override
    public Map<Integer,Double> getDefaultPositions() {
        return myDefaultPositions;
    }

    @Override
    public Double getDefaultPosition(int id) {
        if(myDefaultPositions == null || myDefaultPositions.isEmpty()){
            return null;
        }
        return myDefaultPositions.get(id);
    }
    
    @Override
    public ChannelNode getChannelTree(){
        return myNode;
    }
    
    private Map<Integer, String>  buildChannelNames(Robot robot){
        if(robot == null){
            return null;
        }
        Map<JointId,String> nameMap = robot.getJointNames();
        if(nameMap == null){
            return null;
        }
        Map<Integer, String> names = new HashMap(nameMap.size());
        for(Entry<JointId,String> e : nameMap.entrySet()){
            JointId djId = e.getKey();
            String name = e.getValue();
            if(djId == null || name == null){
                continue;
            }
            Joint.Id jId = djId.getJointId();
            Integer id = jId.getLogicalJointNumber();
            names.put(id, name);
        }
        return names;
    }

    private Map<Integer,Double> buildDefaultPositions(Robot robot) {
        if(robot == null){
            return null;
        }
        RobotPositionMap posMap = robot.getDefaultPositions();
        if(posMap == null){
            return null;
        }
        Map<Integer, Double> pos = new HashMap(posMap.size());
        for(Entry<JointId,NormalizedDouble> e : posMap.entrySet()){
            JointId djId = e.getKey();
            NormalizedDouble val = e.getValue();
            if(djId == null || val == null){
                continue;
            }
            Joint.Id jId = djId.getJointId();
            Integer id = jId.getLogicalJointNumber();
            pos.put(id, val.getValue());
        }
        return pos;
    }
    
    private ChannelNode buildChannelTree(JointGroup<?,?,?> group){
        DefaultChannelNode root = new DefaultChannelNode();
        if(group == null){
            return root;
        }
        root.setName(group.getName());
        for(Joint j : group.getJoints()){
            if(j == null){
                continue;
            }
            root.addChannel(new DefaultChannelDefinition(
                    j.getId().getLogicalJointNumber(), 
                    j.getName() ,
                    j.getDefaultPosition()));
        }
        for(JointGroup jg : group.getJointGroups()){
            root.addGroup(buildChannelTree(jg));
        }
        return root;
    }    
}
